/**
 *     Copyright (C) 2009-2011  Jack A. Rider All rights reserved.
 * 
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 */
 
 package org.xmlportletfactory.portal.goodclient;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.Event;
import javax.portlet.EventRequest;
import javax.portlet.EventResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletURL;
import javax.portlet.ProcessAction;
import javax.portlet.ProcessEvent;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.xml.namespace.QName;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.portlet.PortletFileUpload;

import org.xmlportletfactory.portal.goodclient.model.users;
import org.xmlportletfactory.portal.goodclient.model.impl.usersImpl;
import org.xmlportletfactory.portal.goodclient.service.usersLocalServiceUtil;


import com.liferay.portal.kernel.cache.MultiVMPoolUtil;
import com.liferay.portal.kernel.dao.orm.Criterion;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.DynamicQueryFactoryUtil;
import com.liferay.portal.kernel.dao.orm.PropertyFactoryUtil;
import com.liferay.portal.kernel.dao.orm.RestrictionsFactoryUtil;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.servlet.SessionErrors;
import com.liferay.portal.kernel.servlet.SessionMessages;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.model.User;
import com.liferay.portal.security.permission.PermissionChecker;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.ServiceContextFactory;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portlet.documentlibrary.model.DLFileEntry;
import com.liferay.portlet.documentlibrary.model.DLFolder;
import com.liferay.portlet.documentlibrary.service.DLFileEntryLocalServiceUtil;
import com.liferay.portlet.documentlibrary.service.DLFolderLocalServiceUtil;
import com.liferay.portlet.imagegallery.model.IGFolder;
import com.liferay.portlet.imagegallery.model.IGImage;
import com.liferay.portlet.imagegallery.service.IGFolderLocalServiceUtil;
import com.liferay.portlet.imagegallery.service.IGImageLocalServiceUtil;
import com.liferay.util.bridges.mvc.MVCPortlet;

/**
 * Portlet implementation class Users
 */
public class UsersPortlet extends MVCPortlet {



	public void init() throws PortletException {

		// Edit Mode Pages
		editJSP = getInitParameter("edit-jsp");

		// Help Mode Pages
		helpJSP = getInitParameter("help-jsp");

		// View Mode Pages
		viewJSP = getInitParameter("view-jsp");

		// View Mode Edit users
		editusersJSP = getInitParameter("edit-users-jsp");
	}

	protected void include(String path, RenderRequest renderRequest,
			RenderResponse renderResponse) throws IOException, PortletException {

		PortletRequestDispatcher portletRequestDispatcher = getPortletContext()
				.getRequestDispatcher(path);

		if (portletRequestDispatcher == null) {
			// do nothing
			// _log.error(path + " is not a valid include");
		} else {
			portletRequestDispatcher.include(renderRequest, renderResponse);
		}
	}

	public void doView(RenderRequest renderRequest,
			RenderResponse renderResponse) throws IOException, PortletException {

		String jsp = (String) renderRequest.getParameter("view");
		if (jsp == null || jsp.equals("")) {
			showViewDefault(renderRequest, renderResponse);
		} else if (jsp.equalsIgnoreCase("editUsers")) {
			try {
				showViewEditUsers(renderRequest, renderResponse);
			} catch (Exception ex) {
				_log.debug(ex);
				try {
					showViewDefault(renderRequest, renderResponse);
				} catch (Exception ex1) {
					_log.debug(ex1);
				}
			}
		}
	}

	public void doEdit(RenderRequest renderRequest,
			RenderResponse renderResponse) throws IOException, PortletException {

		showEditDefault(renderRequest, renderResponse);
	}

	public void doHelp(RenderRequest renderRequest,
			RenderResponse renderResponse) throws IOException, PortletException {

		include(helpJSP, renderRequest, renderResponse);
	}

	@SuppressWarnings("unchecked")
	public void showViewDefault(RenderRequest renderRequest,
			RenderResponse renderResponse) throws IOException, PortletException {

		ThemeDisplay themeDisplay = (ThemeDisplay) renderRequest
				.getAttribute(WebKeys.THEME_DISPLAY);

		long groupId = themeDisplay.getScopeGroupId();

		PermissionChecker permissionChecker = themeDisplay
				.getPermissionChecker();

		boolean hasAddPermission = permissionChecker.hasPermission(groupId,
				"org.xmlportletfactory.portal.goodclient.model", groupId, "ADD_USERS");

		List<users> tempResults = Collections.EMPTY_LIST;
		try {
			String orderByType = renderRequest.getParameter("orderByType");
			String orderByCol  = renderRequest.getParameter("orderByCol");
			OrderByComparator comparator = UsersComparator.getUsersOrderByComparator(orderByCol,orderByType);
			MultiVMPoolUtil.clear();

			String usersFilter = ParamUtil.getString(renderRequest, "usersFilter");
			if (usersFilter.equalsIgnoreCase("")) {
				tempResults = usersLocalServiceUtil.findAllInGroup(groupId, comparator);
			} else {
				DynamicQuery query = DynamicQueryFactoryUtil.forClass(users.class)
				.add(
					PropertyFactoryUtil.forName("userName").like("%"+ParamUtil.getString(renderRequest, "usersFilter")+"%")
				);
				tempResults = usersLocalServiceUtil.dynamicQuery(query, -1, -1, comparator);
			}
		
		} catch (Exception e) {
			_log.debug(e);
		}
		renderRequest.setAttribute("tempResults", tempResults);
		renderRequest.setAttribute("hasAddPermission", hasAddPermission);

		PortletURL addUsersURL = renderResponse.createActionURL();
		addUsersURL.setParameter("javax.portlet.action", "newUsers");
		renderRequest.setAttribute("addUsersURL", addUsersURL.toString());

		PortletURL usersFilterURL = renderResponse.createRenderURL();
		usersFilterURL.setParameter("javax.portlet.action", "doView");
		renderRequest.setAttribute("usersFilterURL", usersFilterURL.toString());

		include(viewJSP, renderRequest, renderResponse);
	}

	public void showViewEditUsers(RenderRequest renderRequest, RenderResponse renderResponse) throws Exception {

		SimpleDateFormat formatDia    = new SimpleDateFormat("dd");
		SimpleDateFormat formatMes    = new SimpleDateFormat("MM");
		SimpleDateFormat formatAno    = new SimpleDateFormat("yyyy");
		SimpleDateFormat formatHora   = new SimpleDateFormat("HH");
		SimpleDateFormat formatMinuto = new SimpleDateFormat("mm");

		PortletURL editUsersURL = renderResponse.createActionURL();
		String editType = (String) renderRequest.getParameter("editType");
		if (editType.equalsIgnoreCase("edit")) {
			editUsersURL.setParameter("javax.portlet.action", "updateUsers");
			long usersId = Long.parseLong(renderRequest.getParameter("usersId"));
			users users = usersLocalServiceUtil.getusers(usersId);
			String userName = users.getUserName()+"";
			renderRequest.setAttribute("userName", userName);
			renderRequest.setAttribute("userDateDia", formatDia.format(users.getUserDate()));
			renderRequest.setAttribute("userDateMes", formatMes.format(users.getUserDate()));
			renderRequest.setAttribute("userDateAno", formatAno.format(users.getUserDate()));
			String userDate = dateToJsp(renderRequest, users.getUserDate());
			renderRequest.setAttribute("userDate", userDate);
            renderRequest.setAttribute("users", users);
		} else {
			editUsersURL.setParameter("javax.portlet.action", "addUsers");
			users errorUsers = (users) renderRequest.getAttribute("errorUsers");
			if (errorUsers != null) {
				if (editType.equalsIgnoreCase("update")) {
					editUsersURL.setParameter("javax.portlet.action", "updateUsers");
                }
				renderRequest.setAttribute("users", errorUsers);
                renderRequest.setAttribute("userDateDia", formatDia.format(errorUsers.getUserDate()));
                renderRequest.setAttribute("userDateMes", formatMes.format(errorUsers.getUserDate()));
                renderRequest.setAttribute("userDateAno", formatAno.format(errorUsers.getUserDate()));
				String userDate = dateToJsp(renderRequest,errorUsers.getUserDate());
				renderRequest.setAttribute("userDate", userDate);
			} else {
				usersImpl blankUsers = new usersImpl();
				blankUsers.setUsersId(0);
				blankUsers.setUserName("");
				blankUsers.setUserDate(new Date());
				renderRequest.setAttribute("userDateDia", formatDia.format(blankUsers.getUserDate()));
				renderRequest.setAttribute("userDateMes", formatMes.format(blankUsers.getUserDate()));
				renderRequest.setAttribute("userDateAno", formatAno.format(blankUsers.getUserDate()));
				String userDate = dateToJsp(renderRequest, blankUsers.getUserDate());
				renderRequest.setAttribute("userDate", userDate);
				renderRequest.setAttribute("users", blankUsers);
			}

		}

		ThemeDisplay themeDisplay = (ThemeDisplay) renderRequest.getAttribute(WebKeys.THEME_DISPLAY);


		renderRequest.setAttribute("editUsersURL", editUsersURL.toString());

		include(editusersJSP, renderRequest, renderResponse);
	}

	private String dateToJsp(ActionRequest request, Date date) {
		PortletPreferences prefs = request.getPreferences();
		return dateToJsp(prefs, date);
	}
	private String dateToJsp(RenderRequest request, Date date) {
		PortletPreferences prefs = request.getPreferences();
		return dateToJsp(prefs, date);
	}
	private String dateToJsp(PortletPreferences prefs, Date date) {
		SimpleDateFormat format = new SimpleDateFormat(prefs.getValue("users-date-format", "yyyy/MM/dd"));
		String stringDate = format.format(date);
		return stringDate;
	}
	private String dateTimeToJsp(Date date) {
		SimpleDateFormat format = new SimpleDateFormat("yyyy/MM/dd HH:mm");
		String stringDate = format.format(date);
		return stringDate;
	}

	public void showEditDefault(RenderRequest renderRequest,
			RenderResponse renderResponse) throws PortletException, IOException {

		include(editJSP, renderRequest, renderResponse);
	}

	/* Portlet Actions */

	@ProcessAction(name = "newUsers")
	public void newUsers(ActionRequest request, ActionResponse response) {
		response.setRenderParameter("view", "editUsers");
		response.setRenderParameter("editType", "add");
	}

	@ProcessAction(name = "addUsers")
	public void addUsers(ActionRequest request, ActionResponse response) throws Exception {
            ArrayList<String> errors = new ArrayList();
            users users = usersFromRequest(request);
            if (UsersValidator.validateUsers(users, errors)) {
				usersLocalServiceUtil.addusers(users);
                MultiVMPoolUtil.clear();
                response.setRenderParameter("view", "");
                SessionMessages.add(request, "users-added-successfully");
            } else {
                for (String error : errors) {
                        SessionErrors.add(request, error);
                }
                response.setRenderParameter("view", "editUsers");
                response.setRenderParameter("editType", "add");
                request.setAttribute("errorUsers", users);
            }
	}

	@ProcessAction(name = "eventUsers")
	public void eventUsers(ActionRequest request, ActionResponse response)
			throws Exception {
		long key = ParamUtil.getLong(request, "resourcePrimKey");
		if (Validator.isNotNull(key)) {
		}
	}

	@ProcessAction(name = "editUsers")
	public void editUsers(ActionRequest request, ActionResponse response)
			throws Exception {
		long key = ParamUtil.getLong(request, "resourcePrimKey");
		if (Validator.isNotNull(key)) {
			response.setRenderParameter("usersId", Long.toString(key));
			response.setRenderParameter("view", "editUsers");
			response.setRenderParameter("editType", "edit");
		}
	}

	@ProcessAction(name = "deleteUsers")
	public void deleteUsers(ActionRequest request, ActionResponse response)throws Exception {
		long id = ParamUtil.getLong(request, "resourcePrimKey");
		if (Validator.isNotNull(id)) {
			users users = usersLocalServiceUtil.getusers(id);
			usersLocalServiceUtil.deleteusers(users);
            MultiVMPoolUtil.clear();
			SessionMessages.add(request, "users-deleted-successfully");
		} else {
			SessionErrors.add(request, "users-error-deleting");
		}
	}

	@ProcessAction(name = "updateUsers")
	public void updateUsers(ActionRequest request, ActionResponse response) throws Exception {
            ArrayList<String> errors = new ArrayList();
            users users = usersFromRequest(request);
            if (UsersValidator.validateUsers(users, errors)) {
                usersLocalServiceUtil.updateusers(users);
                MultiVMPoolUtil.clear();
                response.setRenderParameter("view", "");
                SessionMessages.add(request, "users-updated-successfully");
            } else {
                for (String error : errors) {
                        SessionErrors.add(request, error);
                }
				response.setRenderParameter("usersId)",Long.toString(users.getPrimaryKey()));
				response.setRenderParameter("view", "editUsers");
				response.setRenderParameter("editType", "update");
				request.setAttribute("errorUsers", users);
            }
        }

	@ProcessAction(name = "setUsersPref")
	public void setUsersPref(ActionRequest request, ActionResponse response) throws Exception {

		String rowsPerPage = ParamUtil.getString(request, "users-rows-per-page");
		String dateFormat = ParamUtil.getString(request, "users-date-format");
		String datetimeFormat = ParamUtil.getString(request, "users-datetime-format");

		ArrayList<String> errors = new ArrayList();
		if (UsersValidator.validateEditUsers(rowsPerPage, dateFormat, datetimeFormat, errors)) {
			response.setRenderParameter("users-rows-per-page", "");
			response.setRenderParameter("users-date-format", "");
			response.setRenderParameter("users-datetime-format", "");

			PortletPreferences prefs = request.getPreferences();
			prefs.setValue("users-rows-per-page", rowsPerPage);
			prefs.setValue("users-date-format", dateFormat);
			prefs.setValue("users-datetime-format", datetimeFormat);
			prefs.store();

			SessionMessages.add(request, "users-prefs-success");
		}
	}

	private users usersFromRequest(ActionRequest request) {
		ThemeDisplay themeDisplay = (ThemeDisplay) request.getAttribute(WebKeys.THEME_DISPLAY);
		usersImpl users = new usersImpl();
        try {
            users.setUsersId(ParamUtil.getLong(request, "usersId"));
        } catch (Exception nfe) {
		    //Controled en Validator
        }
		users.setUserName(ParamUtil.getString(request, "userName"));
	    PortletPreferences prefs = request.getPreferences();
        SimpleDateFormat format = new SimpleDateFormat(prefs.getValue("users-date-format", "yyyy/MM/dd"));
        int userDateAno = ParamUtil.getInteger(request, "userDateAno");
        int userDateMes = ParamUtil.getInteger(request, "userDateMes")+1;
        int userDateDia = ParamUtil.getInteger(request, "userDateDia");
        try {
			SimpleDateFormat formatter = new SimpleDateFormat("yyyy/MM/dd");
            users.setUserDate(formatter.parse(userDateAno + "/" + userDateMes + "/" + userDateDia));
        } catch (ParseException e) {
			users.setUserDate(new Date());
        }
		try {
		    users.setPrimaryKey(ParamUtil.getLong(request,"resourcePrimKey"));
		} catch (NumberFormatException nfe) {
			//Controled en Validator
        }
		users.setCompanyId(themeDisplay.getCompanyId());
		users.setGroupId(themeDisplay.getScopeGroupId());
		users.setUserId(themeDisplay.getUserId());
		return users;
	}

	protected String editusersJSP;
	protected String editJSP;
	protected String helpJSP;
	protected String viewJSP;

	private static Log _log = LogFactoryUtil.getLog(UsersPortlet.class);

}
